////////////////////////////////////////////
//         ___ ____ _________________     //
//        / _/_  _// _______________/     //
//       / _/ / / / /  ___ ___ ____       //
//      /__/ /_/ / / /   // _/_  _/       //
//     _________/ / / / // _/ / /         //
//  __(c) 1998___/ /___//_/  /_/          //
//                                        //
////////////////////////////////////////////
//          all rights reserved           //
////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// CLayoutMgr implementation
//
// A class for smart layouting of Dialogs (inspired by GridBagLayout from 
// Java(tm))
//
// USAGE: See DialogMgr.html
//
// AUTHOR: Erwin Tratar <tr@et-soft.de>
//
// LEGAL: You may use and modify this code freely as long (1) this copyright 
// notice remains unmodified (2) you submit all changes to the original
// author (3) give credits to the author in your Documentation or AboutBox
// by adding the following line: "Portions copyright 1998 Erwin Tratar/ET-Soft"
// and (4) mark your changes (so I do not get blamed for your bugs)
//
// THIS SOFTWARE COMES WITH ABSOLUTLY NO WARRANTY NEITHER EXPRESSED NOR
// IMPLIED. USE IT AT YOUR OWN RISK. IN NO WAY SHALL THE AUTHER BE LIABLE FOR
// ANY HARM DONE.
//
//
// HISTORY: 
// 1998/05/1	Initial Release
// 1998/05/13	Added ability to have a Pane with a control
// 1998/05/13	Added better support for TabControls
// 1998/05/14	automatically set Icon to IDR_MAINFRAME
// 1998/05/19	no flicker on restoring position in OnInitialUpdate
//				Changed procedure for load/save, see constructor
// 1998/10/02	Added support for Maximum (tracking) size
// 1998/10/02	Much improved handling regarding RELATIVE/GREEDY
//              /w critical minimum size
// 1998/10/02	turn on/off gripper at lower right corner
// 1998/10/05   Support for user defined minimum size for items
//              (was hardcoded 5 before)
// 1998/10/07   Fix for FormViews
// 1998/10/31	Support for SECDialogBar/CDialogBar
// 1998/10/31	simplified interface
// 1998/10/31	Advanced positioning options
// 1998/10/31	Added paneNull for empty Pane (former: NULL)
// 1998/11/20	Swapped CDialogMgr constructor parameters
// 1998/11/20	Added CPane::addItemSpaceBetween 
//				(Leo Zelevinsky)
// 1998/11/24	Added fixup for greedy panes
// 1998/11/24	addItemSpaceBetween now subtracts 2*nDefaultBorder
// 1998/11/24	addGrowing() added as a shortcut for a paneNull
// 1998/11/24	simplified interface: no more CPaneBase:: / CPane:: 
//				needed
// 1998/11/24	added FILL_* Modes
// 1998/11/24	improved maximum size handling for greedy panes
// 1998/11/25	Fixup of greedy panes caused infinite loop in some 
//				cases
// 1999/01/07	addItemSpaceLike() added
// 1999/04/03   Fixed CFormViewMgr memory leak

// TODO:
// - Split Layout code into understandable pieces
// - Take minimum tracking size of item from current size
// - think about allowing empty panes

#define OEMRESOURCE
#include	<windows.h>

#include "stdafx.h"
#include "DialogMgr.h"

#ifndef OBM_SIZE
#define	OBM_SIZE		32766
// taken from WinresRc.h
// if not used for any reason
#endif


#ifdef _AUTO_SET_ICON
#include "resource.h"
#endif

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

// the _NULL-Pane
CWnd* CLayoutMgr::paneNull = 0;

void CLayoutMgr::Layout(CRect& rcClient)
{
	// Does nothing, use WITH_LAYOUT in derived class
	// see DialogMgr.h
}

void CLayoutMgr::UpdateLayout()
{
	if(!m_pRootPane)
		return;

	// Check constraints
	CRect rcClient = GetRect();

	if( m_pWnd->IsKindOf( RUNTIME_CLASS( CDialog ) ) ) {
		CRect rcWindow;
		m_pWnd->GetWindowRect(rcWindow);

		CRect rcBorder = rcWindow;
		rcBorder -= rcClient;

		// Min and Max info
		int minWidth	= m_pRootPane->getMinConstrainHorz() + rcBorder.Width()  + 2*m_sizeRootBorders.cx;
		int minHeight	= m_pRootPane->getMinConstrainVert() + rcBorder.Height() + 2*m_sizeRootBorders.cy;
		int maxWidth	= m_pRootPane->getMaxConstrainHorz();
		if(maxWidth != -1) {
			maxWidth += rcBorder.Width()  + 2*m_sizeRootBorders.cx;
			maxWidth = max(maxWidth, minWidth);
		}
		int maxHeight	= m_pRootPane->getMaxConstrainVert();
		if(maxHeight != -1) {
			maxHeight += rcBorder.Height() + 2*m_sizeRootBorders.cy;
			maxHeight = max(maxHeight, minHeight);
		}

		if(rcWindow.Width() < minWidth)
			rcWindow.right = rcWindow.left + minWidth;
		if(rcWindow.Height() < minHeight)
			rcWindow.bottom = rcWindow.top + minHeight;

		if(maxWidth != -1  && rcWindow.Width() > maxWidth)
			rcWindow.right = rcWindow.left + maxWidth;
		if(maxHeight != -1 && rcWindow.Height() > maxHeight)
			rcWindow.bottom = rcWindow.top + maxHeight;

		m_pWnd->MoveWindow(rcWindow);
	}
	// Do the Layout
	rcClient = GetRect();

	// Add a Border around the rootPane
	rcClient.top	+= m_sizeRootBorders.cy;
	rcClient.bottom -= m_sizeRootBorders.cy;
	rcClient.left	+= m_sizeRootBorders.cx;
	rcClient.right	-= m_sizeRootBorders.cx;

	if(GetWnd()->IsWindowVisible()) {
		// Avoid ugly artifacts
		GetWnd()->SetRedraw(FALSE);
		Layout(rcClient);
		GetWnd()->SetRedraw(TRUE);
	}
	else
		Layout(rcClient);

	GetWnd()->Invalidate();
}

bool CLayoutMgr::Save(LPCTSTR lpstrRegKey)
{
	CRect rcWnd;

	if( IsWindow( GetWnd()->m_hWnd ) ) {
		GetWnd()->GetWindowRect(rcWnd);
		if(rcWnd.right >= 0 && rcWnd.bottom >= 0 && rcWnd.Height()>0 && rcWnd.Width() > 0) {
			AfxGetApp()->WriteProfileInt( lpstrRegKey, "PosX", rcWnd.left );
			AfxGetApp()->WriteProfileInt( lpstrRegKey, "PosY", rcWnd.top );
			AfxGetApp()->WriteProfileInt( lpstrRegKey, "SizeX", rcWnd.Width() );
			AfxGetApp()->WriteProfileInt( lpstrRegKey, "SizeY", rcWnd.Height() );
		}
	}
	return true;
}

bool CLayoutMgr::Load(LPCTSTR lpstrRegKey)
{
	// default to current size
	CRect rcWnd;
	GetWnd()->GetWindowRect(rcWnd);

	rcWnd.left	= AfxGetApp()->GetProfileInt( lpstrRegKey, "PosX", rcWnd.left );
	rcWnd.top	= AfxGetApp()->GetProfileInt( lpstrRegKey, "PosY", rcWnd.top );
	rcWnd.right	= rcWnd.left + AfxGetApp()->GetProfileInt( lpstrRegKey, "SizeX", rcWnd.Width() );
	rcWnd.bottom= rcWnd.top  + AfxGetApp()->GetProfileInt( lpstrRegKey, "SizeY", rcWnd.Height() );

	GetWnd()->MoveWindow(rcWnd);
	return true;
}


/////////////////////////////////////////////////////////////////////////////
// CLayoutMgr::CPaneItem implementation


CLayoutMgr::CPaneItem::CPaneItem(CWnd* pWnd, CLayoutMgr* pMgr, DWORD modeResize/*=GREEDY*/
								 , int sizeX/*=0*/, int sizeY/*=0*/
								 , int sizeXMin/*=0*/, int sizeYMin/*=0*/ ) : CPaneBase( pMgr )
{
	m_modeResize	= modeResize;
	m_hwndCtrl		= pWnd->GetSafeHwnd();

	m_sizeX			= 0;
	m_sizeY			= 0;


	m_sizeXMin		= sizeXMin;
	m_sizeYMin		= sizeYMin;

	if(!m_hwndCtrl) {			// only Dummy!
		m_sizeX = sizeX;
		m_sizeY = sizeY;
	}
	else {
		CRect rcControl;
		::GetWindowRect(m_hwndCtrl, &rcControl);

		if(sizeX == 0) {
			m_sizeX			= rcControl.Width();
		}
		else {
			m_sizeX = sizeX;
		}
		if( m_sizeXMin == -1 ) {
			// do not make smaller than current size
			m_sizeXMin		= rcControl.Width();
		}

		if(sizeY == 0) {
			m_sizeY			= rcControl.Height();
		}
		else {
			m_sizeY = sizeY;
		}
		if( m_sizeYMin == -1 ) {
			// do not make smaller than current size
			m_sizeYMin		= rcControl.Height();
		}
	}
}

CLayoutMgr::CPaneItem::CPaneItem( UINT nID, CLayoutMgr* pMgr, DWORD modeResize/*=GREEDY*/
								 , int sizeX/*=0*/, int sizeY/*=0*/
								 , int sizeXMin/*=0*/, int sizeYMin/*=0*/ ) : CPaneBase( pMgr )
{
	CWnd* pWnd		= pMgr->GetWnd()->GetDlgItem(nID);
	m_hwndCtrl		= pWnd->GetSafeHwnd();

	m_sizeX			= 0;
	m_sizeY			= 0;
	m_modeResize	= modeResize;

	m_sizeXMin = sizeXMin;
	m_sizeYMin = sizeYMin;

	if(!m_hwndCtrl) {			// only Dummy!
		m_sizeX = sizeX;
		m_sizeY = sizeY;
	}
	else {
		CRect rcControl;
		::GetWindowRect(m_hwndCtrl, &rcControl);

		if(sizeX == 0) {
			m_sizeX			= rcControl.Width();
		}
		else {
			m_sizeX = sizeX;
		}
		if( m_sizeXMin == -1 ) {
			// do not make smaller than current size
			m_sizeXMin		= rcControl.Width();
		}

		if(sizeY == 0) {
			m_sizeY			= rcControl.Height();
		}
		else {
			m_sizeY = sizeY;
		}
		if( m_sizeYMin == -1 ) {
			// do not make smaller than current size
			m_sizeYMin		= rcControl.Height();
		}
	}
}

int CLayoutMgr::CPaneItem::getConstrainHorz(int sizeParent) 
{
	if( m_modeResize & ABSOLUTE_HORZ) {
		return m_sizeX;	
	}
	if(m_modeResize & RELATIVE_HORZ) {
		return (sizeParent * m_sizeX) / 100;	
	}
	return -1;
}

int CLayoutMgr::CPaneItem::getConstrainVert(int sizeParent) 
{
	if(m_modeResize & ABSOLUTE_VERT) {
		return m_sizeY;	
	}
	if(m_modeResize & RELATIVE_VERT) {
		return (sizeParent * m_sizeY) / 100;	
	}
	return -1;
}

int CLayoutMgr::CPaneItem::getMinConstrainHorz() 
{
	if(m_modeResize & ABSOLUTE_HORZ) {
		return m_sizeX;	
	}
	return max(nMinConstrain, m_sizeXMin);
}

int CLayoutMgr::CPaneItem::getMinConstrainVert() 
{
	if(m_modeResize & ABSOLUTE_VERT) {
		return m_sizeY;	
	}
	return max(nMinConstrain, m_sizeYMin);
}

int CLayoutMgr::CPaneItem::getMaxConstrainHorz() 
{
	if(m_modeResize & ABSOLUTE_HORZ) {
		return m_sizeX;	
	}
	return -1;
}

int CLayoutMgr::CPaneItem::getMaxConstrainVert() 
{
	if(m_modeResize & ABSOLUTE_VERT) {
		return m_sizeY;	
	}
	return -1;	
}

bool CLayoutMgr::CPaneItem::resizeTo(CRect& rcNewArea) 
{
	if(m_hwndCtrl) {

		CRect rcWnd;
		::GetWindowRect( m_hwndCtrl, rcWnd );

		if( !(m_modeResize & ALIGN_FILL_HORZ) && m_modeResize & ABSOLUTE_HORZ ) {

			rcNewArea.right = rcNewArea.left + rcWnd.Width();

			if( (m_modeResize & ALIGN_HCENTER) == ALIGN_HCENTER ) {
				rcNewArea.OffsetRect( (rcWnd.Width() - rcNewArea.Width())/2, 0 ); 
			}
			else if( m_modeResize & ALIGN_RIGHT ) {
				rcNewArea.OffsetRect( rcWnd.Width() - rcNewArea.Width(), 0 ); 
			}
		}
		if( !(m_modeResize & ALIGN_FILL_VERT) && m_modeResize & ABSOLUTE_VERT ) {

			rcNewArea.bottom = rcNewArea.top + rcWnd.Height();

			if( (m_modeResize & ALIGN_VCENTER) == ALIGN_VCENTER ) {
				rcNewArea.OffsetRect( 0, (rcWnd.Height() - rcNewArea.Height())/2 ); 
			}
			else if( m_modeResize & ALIGN_BOTTOM ) {
				rcNewArea.OffsetRect( 0, rcWnd.Height() - rcNewArea.Height() ); 
			}


		}
		::MoveWindow( m_hwndCtrl, rcNewArea.left, rcNewArea.top, rcNewArea.Width(), rcNewArea.Height(), true );
	}
	return true;
}


/////////////////////////////////////////////////////////////////////////////
// CLayoutMgr::CPaneTab implementation


CLayoutMgr::CPaneTab::CPaneTab( CTabCtrl* pTab, CLayoutMgr* pMgr, layOrientation orientation, int sizeBorder /*= nDefaultBorder*/, int sizeExtraBorder /*= 0*/ )
: CLayoutMgr::CPane(pMgr, orientation, sizeBorder, sizeExtraBorder) 
{
	ASSERT(pTab);
	m_pTab = pTab;
}

int CLayoutMgr::CPaneTab::getConstrainHorz(int sizeParent)
{
	CRect rcTab;
	m_pTab->AdjustRect(TRUE, &rcTab);

	if(rcTab.Width() > sizeParent)
		return rcTab.Width();

	return CPane::getConstrainHorz(sizeParent /*- rcTab.Width()*/);
}

int CLayoutMgr::CPaneTab::getConstrainVert(int sizeParent)
{
	CRect rcTab;
	m_pTab->AdjustRect(TRUE, &rcTab);

	if(rcTab.Height() > sizeParent)
		return rcTab.Height();

	return CPane::getConstrainVert(sizeParent /*- rcTab.Height()*/);
}

int CLayoutMgr::CPaneTab::getMinConstrainHorz()
{
	CRect rcTab(0,0,0,0);
	m_pTab->AdjustRect(TRUE, &rcTab);

	return CPane::getMinConstrainHorz() + rcTab.Width() ;
}

int CLayoutMgr::CPaneTab::getMinConstrainVert()
{
	CRect rcTab(0,0,0,0);
	m_pTab->AdjustRect(TRUE, &rcTab);

	return CPane::getMinConstrainVert() + rcTab.Height();
}

int CLayoutMgr::CPaneTab::getMaxConstrainHorz()
{
	CRect rcTab(0,0,0,0);
	m_pTab->AdjustRect(TRUE, &rcTab);

	int paneMax = CPane::getMaxConstrainHorz();
	return (paneMax != -1) ? paneMax + rcTab.Width() : -1;
}

int CLayoutMgr::CPaneTab::getMaxConstrainVert()
{
	CRect rcTab(0,0,0,0);
	m_pTab->AdjustRect(TRUE, &rcTab);

	int paneMax = CPane::getMaxConstrainVert();
	return (paneMax != -1) ? paneMax + rcTab.Height() : -1;
}

bool CLayoutMgr::CPaneTab::resizeTo(CRect& rcNewArea)
{
	m_pTab->MoveWindow(rcNewArea);
	m_pTab->AdjustRect(FALSE,rcNewArea);

	return CPane::resizeTo(rcNewArea);
}

/////////////////////////////////////////////////////////////////////////////
// CLayoutMgr::CPaneCtrl implementation


CLayoutMgr::CPaneCtrl::CPaneCtrl( CWnd* pCtrl, CLayoutMgr* pMgr, layOrientation orientation, int sizeBorder /*= nDefaultBorder*/, int sizeExtraBorder /*= 0*/, int sizeTopExtra /*= 0*/ )
: CLayoutMgr::CPane(pMgr, orientation, sizeBorder, sizeExtraBorder)
{
	m_sizeTopExtra = sizeTopExtra;

	ASSERT(pCtrl);
	m_hwndCtrl = pCtrl->GetSafeHwnd();
}

CLayoutMgr::CPaneCtrl::CPaneCtrl( UINT nID, CLayoutMgr* pMgr, layOrientation orientation, int sizeBorder /*= nDefaultBorder*/, int sizeExtraBorder /*= 0*/, int sizeTopExtra /*= 0*/ )
: CLayoutMgr::CPane(pMgr, orientation, sizeBorder, sizeExtraBorder)
{
	m_sizeTopExtra = sizeTopExtra;

	m_hwndCtrl = ::GetDlgItem(pMgr->GetWnd()->GetSafeHwnd(), nID);
	ASSERT(m_hwndCtrl);
}

int CLayoutMgr::CPaneCtrl::getConstrainHorz(int sizeParent)
{
	return CPane::getConstrainHorz(sizeParent) ;
}

int CLayoutMgr::CPaneCtrl::getConstrainVert(int sizeParent)
{
	return CPane::getConstrainVert(sizeParent);
}

int CLayoutMgr::CPaneCtrl::getMinConstrainHorz()
{
	return CPane::getMinConstrainHorz();
}

int CLayoutMgr::CPaneCtrl::getMinConstrainVert()
{
	return CPane::getMinConstrainVert() + m_sizeTopExtra;
}

int CLayoutMgr::CPaneCtrl::getMaxConstrainHorz()
{
	int paneMax = CPane::getMaxConstrainHorz();
	return ( paneMax == -1) ? -1 : paneMax ;
}

int CLayoutMgr::CPaneCtrl::getMaxConstrainVert()
{
	int paneMax = CPane::getMaxConstrainVert();
	return ( paneMax == -1) ? -1 : paneMax + m_sizeTopExtra;
}

bool CLayoutMgr::CPaneCtrl::resizeTo(CRect& rcNewArea)
{
	::MoveWindow(m_hwndCtrl, rcNewArea.left, rcNewArea.top, rcNewArea.Width(), rcNewArea.Height(), true);

	rcNewArea.top	+= m_sizeTopExtra;
	return CPane::resizeTo(rcNewArea);
}

/////////////////////////////////////////////////////////////////////////////
// CLayoutMgr::CPane implementation

CLayoutMgr::CPane::CPane( CLayoutMgr* pMgr, layOrientation orientation, int sizeBorder /* = nDefaultBorder */, int sizeExtraBorder /*= 0*/) 
: CPaneBase(pMgr)
{
	m_Orientation	= orientation;
	m_sizeBorder	= sizeBorder;
	m_sizeSecondary	= 0;
	m_modeResize	= 0;
	m_sizeExtraBorder= sizeExtraBorder;
}


CLayoutMgr::CPane::~CPane() 
{
	// Destroy all items
	for(int i=0; i<m_paneItems.GetSize(); delete m_paneItems[i++]);
	m_paneItems.RemoveAll();
}


bool CLayoutMgr::CPane::addItem( CWnd* pWnd, DWORD modeResize /*=GREEDY*/, int sizeX /*=0*/, int sizeY /*=0*/, int sizeXMin /*=0*/, int sizeYMin /*=0*/)
{
	CPaneItem* pItem = new CPaneItem( pWnd, m_pMgr, modeResize, sizeX, sizeY, sizeXMin, sizeYMin);
	return addPane( pItem );
}

bool CLayoutMgr::CPane::addItem( UINT nID, DWORD modeResize /*=GREEDY*/, int sizeX /*=0*/, int sizeY /*=0*/, int sizeXMin /*=0*/, int sizeYMin /*=0*/)
{
	CPaneItem* pItem = new CPaneItem( nID, m_pMgr, modeResize, sizeX, sizeY, sizeXMin, sizeYMin);
	return addPane( pItem );
}

bool CLayoutMgr::CPane::addGrowing()
{
	CPaneItem* pNewItem = 0;
	if( m_Orientation == HORIZONTAL ) {
		pNewItem = new CPaneItem(paneNull, m_pMgr, ABSOLUTE_VERT,0,0);
	}
	else {
		pNewItem = new CPaneItem(paneNull, m_pMgr, ABSOLUTE_HORZ,0,0);
	}

	ASSERT( pNewItem );
	return addPane( pNewItem );
}

bool CLayoutMgr::CPane::addItemSpaceBetween( CWnd* pWndFirst, CWnd* pWndSecond )
{
	CPaneItem* pNewItem = 0;

	if( m_Orientation == HORIZONTAL ) {
		// I'm interested in horizontal spacing

		CRect rLeft, rRight;
		pWndFirst->GetWindowRect(&rLeft);
		pWndSecond->GetWindowRect(&rRight);

		int sizeX = rRight.left - rLeft.right;
	
		if( sizeX < 0 ) {
			// compare top to top
			sizeX = rRight.left - rLeft.left;
		}
		else {
			sizeX -= 2*nDefaultBorder;
		}

		pNewItem = new CPaneItem(paneNull, m_pMgr, ABSOLUTE_HORZ, sizeX, 0);
	}
	else {
		// I'm interested in vertical spacing
		CRect rTop, rBot;
		pWndFirst->GetWindowRect(&rTop);
		pWndSecond->GetWindowRect(&rBot);

		int sizeY = rBot.top - rTop.bottom;

		if( sizeY < 0 ) {
			// compare top to top
			sizeY = sizeY = rBot.top - rTop.top;
		}
		else {
			sizeY -= 2*nDefaultBorder;
		}

		pNewItem = new CPaneItem(paneNull, m_pMgr, ABSOLUTE_VERT, 0, sizeY);
	}

	ASSERT( pNewItem );
	return addPane( pNewItem );
}

bool CLayoutMgr::CPane::addItemSpaceBetween( UINT nIDFirst, UINT nIDSecond )
{
	CWnd *pFirst	= m_pMgr->GetWnd()->GetDlgItem(nIDFirst);
	CWnd *pSecond	= m_pMgr->GetWnd()->GetDlgItem(nIDSecond);

	ASSERT( pFirst && pSecond );

	return addItemSpaceBetween( pFirst, pSecond );
}

bool CLayoutMgr::CPane::addItemSpaceLike( CWnd* pWnd )
{
	CPaneItem* pNewItem = 0;

	CRect rRect;
	pWnd->GetWindowRect(&rRect);

	if( m_Orientation == HORIZONTAL ) {
		// I'm interested in horizontal spacing
		pNewItem = new CPaneItem(paneNull, m_pMgr, ABSOLUTE_HORZ, rRect.Width(), 0);
	}
	else {
		// I'm interested in vertical spacing
		pNewItem = new CPaneItem(paneNull, m_pMgr, ABSOLUTE_VERT, 0, rRect.Height() );
	}

	ASSERT( pNewItem );
	return addPane( pNewItem );
}

bool CLayoutMgr::CPane::addItemSpaceLike( UINT nID )
{
	CWnd *pWnd	= m_pMgr->GetWnd()->GetDlgItem(nID);
	ASSERT( pWnd );

	return addItemSpaceLike( pWnd );
}

bool CLayoutMgr::CPane::addPane( CPane* pSubpane, DWORD modeResize, int sizeSecondary /* = 0 */) 
{
	if( pSubpane->getOrientation() == m_Orientation)
		return false;

	pSubpane->m_modeResize = modeResize;

	if(m_Orientation==HORIZONTAL && (modeResize & ABSOLUTE_HORZ) ) {
		if(sizeSecondary == 0) {
			pSubpane->m_sizeSecondary = pSubpane->getMinConstrainHorz();
		}
	}
	else if(m_Orientation==HORIZONTAL && (modeResize & RELATIVE_HORZ) ) {
		pSubpane->m_sizeSecondary = max( nMinConstrain, sizeSecondary);
	}
	else if(m_Orientation==VERTICAL && (modeResize & ABSOLUTE_VERT) ) {
		if(sizeSecondary == 0) {
			pSubpane->m_sizeSecondary = pSubpane->getMinConstrainVert();
		}
	}
	else if(m_Orientation==VERTICAL && (modeResize & RELATIVE_VERT) ) {
		pSubpane->m_sizeSecondary = max( nMinConstrain, sizeSecondary);
	}

	m_paneItems.Add(pSubpane);

	return true;
}

bool CLayoutMgr::CPane::addPane( CPaneBase* pItem ) 
{
	m_paneItems.Add(pItem);
	return true;
}

int CLayoutMgr::CPane::getConstrainHorz(int sizeParent) 
{
	ASSERT( m_Orientation == VERTICAL);

	if( m_modeResize & RELATIVE_HORZ ) {
		return (sizeParent * m_sizeSecondary) / 100;
	}
	else if( m_modeResize & ABSOLUTE_HORZ ){
		return m_sizeSecondary;
	}
	else
		return 0;
}


int CLayoutMgr::CPane::getConstrainVert(int sizeParent) 
{
	ASSERT( m_Orientation == HORIZONTAL);

	if( m_modeResize & RELATIVE_VERT ) {
		return (sizeParent * m_sizeSecondary) / 100;
	}
	else if( m_modeResize & ABSOLUTE_VERT ) {
		return m_sizeSecondary;
	}
	else {
		return 0;
	}
}

int CLayoutMgr::CPane::getMaxConstrainHorz() 
{
	if(m_Orientation == HORIZONTAL) {
		int nMaxConstr = -1;
		for(int i=0; i<m_paneItems.GetSize(); ++i) {
			CPaneBase* pItem = m_paneItems[i];

			int nConstrain = pItem->getMaxConstrainHorz();
			if(nConstrain == -1)
				return -1;

			nMaxConstr += nConstrain;
		}
		return (nMaxConstr == -1) ? -1 : nMaxConstr + (m_paneItems.GetUpperBound()*m_sizeBorder) + 2*m_sizeExtraBorder;
	}
	else if( m_modeResize & ABSOLUTE_HORZ && m_sizeSecondary!=0) {
		return m_sizeSecondary; // + 2*m_sizeExtraBorder;
	}
	else {
		int nMaxConstr = -1;
		for(int i=0; i<m_paneItems.GetSize(); ++i) {
			CPaneBase* pItem = m_paneItems[i];

			int nConstrain = pItem->getMaxConstrainHorz();

			if( nConstrain == -1)
				return -1;
			else
				nMaxConstr = max(nMaxConstr, nConstrain);

		}
		return (nMaxConstr == -1) ? -1 : nMaxConstr + 2*m_sizeExtraBorder;
	}
}

int CLayoutMgr::CPane::getMaxConstrainVert() 
{
	if(m_Orientation == VERTICAL) {
		int nMaxConstr = -1;
		for(int i=0; i<m_paneItems.GetSize(); ++i) {
			CPaneBase* pItem = m_paneItems[i];

			int nConstrain = pItem->getMaxConstrainVert();
			if(nConstrain == -1)
				return -1;

			nMaxConstr += nConstrain;
		}
		return (nMaxConstr == -1) ? -1 : nMaxConstr + (m_paneItems.GetUpperBound()*m_sizeBorder) + 2*m_sizeExtraBorder;
	}
	else if( m_modeResize & ABSOLUTE_VERT && m_sizeSecondary!=0) {
		return m_sizeSecondary; // + 2*m_sizeExtraBorder;
	}
	else {
		int nMaxConstr = -1;
		for(int i=0; i<m_paneItems.GetSize(); ++i) {
			CPaneBase* pItem = m_paneItems[i];

			int nConstrain = pItem->getMaxConstrainVert();

			if( nConstrain == -1)
				return -1;
			else
				nMaxConstr = max(nMaxConstr, nConstrain);

		}
		return (nMaxConstr == -1) ? -1 : nMaxConstr + 2*m_sizeExtraBorder;
	}
}

int CLayoutMgr::CPane::getMinConstrainHorz() 
{
	if(m_Orientation == HORIZONTAL) {
		int nMaxConstr = 0;
		for(int i=0; i<m_paneItems.GetSize(); ++i) {
			CPaneBase* pItem = m_paneItems[i];
			nMaxConstr += pItem->getMinConstrainHorz();
		}
		return nMaxConstr + (m_paneItems.GetUpperBound()*m_sizeBorder) + 2*m_sizeExtraBorder;
	}
	else if( m_modeResize & ABSOLUTE_HORZ && m_sizeSecondary!=0) {
		return m_sizeSecondary; // + 2*m_sizeExtraBorder;
	}
	else {
		int nMaxConstr = 0;
		for(int i=0; i<m_paneItems.GetSize(); ++i) {
			CPaneBase* pItem = m_paneItems[i];
			int nConstrain = pItem->getMinConstrainHorz();
			nMaxConstr = max(nMaxConstr, nConstrain);
		}
		return nMaxConstr + 2*m_sizeExtraBorder;
	}
}

int CLayoutMgr::CPane::getMinConstrainVert() 
{
	if(m_Orientation == VERTICAL) {
		int nMaxConstr = 0;
		for(int i=0; i<m_paneItems.GetSize(); ++i) {
			CPaneBase* pItem = m_paneItems[i];
			nMaxConstr += pItem->getMinConstrainVert();
		}
		return nMaxConstr + (m_paneItems.GetUpperBound()*m_sizeBorder) + 2*m_sizeExtraBorder;
	}
	else if( m_modeResize & ABSOLUTE_VERT && m_sizeSecondary!=0) {
		return m_sizeSecondary; // + 2*m_sizeExtraBorder;
	}
	else {
		int nMaxConstr = 0;
		for(int i=0; i<m_paneItems.GetSize(); ++i) {
			CPaneBase* pItem = m_paneItems[i];
			int nConstrain = pItem->getMinConstrainVert();
			nMaxConstr = max(nMaxConstr, nConstrain);
		}
		return nMaxConstr + 2*m_sizeExtraBorder;
	}
}

bool CLayoutMgr::CPane::resizeTo(CRect& rcNewArea) 
{
	ASSERT(m_paneItems.GetSize());

	CArray<int,int>	sizePrimary;
	sizePrimary.SetSize(m_paneItems.GetSize());

	CArray<int,int>	sizeMin;
	CArray<int,int>	sizeMax;
	sizeMin.SetSize(m_paneItems.GetSize());
	sizeMax.SetSize(m_paneItems.GetSize());

	// Is it the primary Orientation?
	if( m_Orientation == HORIZONTAL ) {

		int availX = rcNewArea.Width() - (m_paneItems.GetUpperBound()*m_sizeBorder);
		availX -= 2*m_sizeExtraBorder;

		rcNewArea.top		+= m_sizeExtraBorder;
		rcNewArea.bottom	-= m_sizeExtraBorder;

		int nGreedy = 0;
		// first all absolute items
		for(int i=0; i<m_paneItems.GetSize(); ++i) {
			CPaneBase* pItem = m_paneItems[i];
			if(pItem->modeResize() & ABSOLUTE_HORZ) {
				availX -= (sizePrimary[i] = pItem->getConstrainHorz(0));
			}

			// count Greedy items for later
			if(!(pItem->modeResize() & ABSOLUTE_HORZ) && !(pItem->modeResize() & RELATIVE_HORZ)) {
				nGreedy++;
			}

			sizeMin[i] = pItem->getMinConstrainHorz();
			sizeMax[i] = pItem->getMaxConstrainHorz();
		}

		// Must not be negative
		availX = max(availX, 0);

		// Then all relative items
		int availRelX = availX;
		int relDiff = 0;
		int relLeft = 0;
		int relCount = 0;
		for(i=0; i<m_paneItems.GetSize(); ++i) {
			CPaneBase* pItem = m_paneItems[i];
			if(pItem->modeResize() & RELATIVE_HORZ) {
				int nSizeRelMin = sizeMin[i];
				int nSizeRelMax = sizeMax[i];
				int nSizeRel	= pItem->getConstrainHorz(availRelX);

				if( nSizeRel < nSizeRelMin) {
					sizePrimary[i]	= -nSizeRelMin;
					relDiff += ( nSizeRelMin - nSizeRel );
				}
				else if(  nSizeRelMax != -1 && nSizeRel > nSizeRelMax) {
					sizePrimary[i]	= -nSizeRelMax;
					relDiff += ( nSizeRelMax - nSizeRel );
				}
				else {
					relLeft	+= ( nSizeRel - nSizeRelMin );
					sizePrimary[i]	= nSizeRel;
					relCount	+= nSizeRel;
				}
				availX	-= nSizeRel;
			}
		}

		if(relLeft < relDiff && availX >= (relDiff-relLeft) ){
			availX -= (relDiff-relLeft);
			relDiff = relLeft;
		}

		ASSERT( availX >= 0 && relLeft >= relDiff);

		// Fixup Relative
		while( relDiff != 0) {
			ASSERT( relCount);
			int relSaved = 0;
			for(i=0; i<m_paneItems.GetSize(); ++i) {
				CPaneBase* pItem = m_paneItems[i];
				if(pItem->modeResize() & RELATIVE_HORZ && sizePrimary[i] > 0) {

					bool bLast = false;
					int nDiff = (relDiff * sizePrimary[i]) / relCount;
					if( abs(relDiff) <= 5 ) {
						nDiff = relDiff;
						bLast = true;
					}

					int nNewSize = sizePrimary[i] - nDiff;
				
					if( nNewSize < sizeMin[i] ) {
						relSaved += ( sizePrimary[i] - sizeMin[i] );
						sizePrimary[i] = sizeMin[i];
						relCount-= ( sizePrimary[i] );
					}
					else {
						relSaved += ( sizePrimary[i] - nNewSize );
						sizePrimary[i] = nNewSize;
						if(bLast)
							break;
					}
				}
			}
			ASSERT(relSaved != 0);	// Saved some space in every iteration
			relDiff -= relSaved;
		}
		// Fixup Relative - II
		for(i=0; i<m_paneItems.GetSize(); ++i) {
			CPaneBase* pItem = m_paneItems[i];
			if(pItem->modeResize() & RELATIVE_HORZ) {
				if(sizePrimary[i] < 0)
					sizePrimary[i] *= -1;
			}
		}
		

		// lastly all Greedy items
		int greedyDiff = 0;
		int greedyLeft = 0;
		int greedyCount = 0;
		for(i=0; i<m_paneItems.GetSize(); ++i) {
			CPaneBase* pItem = m_paneItems[i];
			if(!(pItem->modeResize() & ABSOLUTE_HORZ) && !(pItem->modeResize() & RELATIVE_HORZ)) {

				int nSize		= availX / nGreedy;
				int nSizeMin	= sizeMin[i];
				int nSizeMax	= sizeMax[i];

				if( i == m_paneItems.GetUpperBound() )
					nSize = availX;						// the last gets the rest
//				else if( nSize < nSizeMin)
//					nSize = nSizeMin;

				if( nSize < nSizeMin) {
					sizePrimary[i]	= -nSizeMin;
					greedyDiff		+= ( nSizeMin - nSize );
				}
				else if( nSizeMax != -1 && nSize > nSizeMax) {
					sizePrimary[i]	= -nSizeMax;
					greedyDiff		+= ( nSizeMax - nSize );
				}
				else {
					greedyLeft		+= ( nSize - nSize );
					sizePrimary[i]	= nSize;
					greedyCount		+= nSize;
				}

				availX			-= nSize;

				--nGreedy;
			}
		}
		// Fixup Greedy
		while( greedyDiff != 0 && greedyCount > 0) {
			int greedySaved = 0;
			for(i=0; i<m_paneItems.GetSize(); ++i) {
				CPaneBase* pItem = m_paneItems[i];
				if(!(pItem->modeResize() & ABSOLUTE_HORZ) && !(pItem->modeResize() & RELATIVE_HORZ) && sizePrimary[i] > 0) {

					bool bLast = false;
					int nDiff = (greedyDiff * sizePrimary[i]) / greedyCount;
					if( abs(greedyDiff) <= 5 ) {
						nDiff = greedyDiff;
						bLast = true;
					}

					int nNewSize = sizePrimary[i] - nDiff;			
					if( nNewSize < sizeMin[i] ) {
						greedySaved += ( sizePrimary[i] - sizeMin[i] );
						sizePrimary[i] = sizeMin[i];
						greedyCount-= ( sizePrimary[i] );
					}
					else {
						greedySaved += ( sizePrimary[i] - nNewSize );
						sizePrimary[i] = nNewSize;
						if(bLast)
							break;
					}
				}
			}
			ASSERT(greedySaved != 0 || greedyCount<=0);	// Saved some space in every iteration
			greedyDiff -= greedySaved;
		}
		// Fixup Greedy - II
		for(i=0; i<m_paneItems.GetSize(); ++i) {
			CPaneBase* pItem = m_paneItems[i];
			if(!(pItem->modeResize() & ABSOLUTE_HORZ) && !(pItem->modeResize() & RELATIVE_HORZ)) {
				if(sizePrimary[i] < 0)
					sizePrimary[i] *= -1;
			}
		}


		// Now reposition all items:
		int nOffset = rcNewArea.left + m_sizeExtraBorder;
		for(i=0; i<m_paneItems.GetSize(); ++i) {
			CPaneBase* pItem = m_paneItems[i];

			CRect rcPane(nOffset, rcNewArea.top, nOffset+sizePrimary[i], rcNewArea.bottom);
			pItem->resizeTo( rcPane );

			nOffset += m_sizeBorder + sizePrimary[i];
		}				
	}
	else {

		int availY = rcNewArea.Height() - (m_paneItems.GetUpperBound()*m_sizeBorder);
		availY-= 2*m_sizeExtraBorder;

		rcNewArea.left	+= m_sizeExtraBorder;
		rcNewArea.right	-= m_sizeExtraBorder;

		int nGreedy = 0;
		// first all absolute items
		for(int i=0; i<m_paneItems.GetSize(); ++i) {
			CPaneBase* pItem = m_paneItems[i];
			if(pItem->modeResize() & ABSOLUTE_VERT) {
				availY -= (sizePrimary[i] = pItem->getConstrainVert(0));
			}

			// count Greedy items for later
			if(!(pItem->modeResize() & ABSOLUTE_VERT) && !(pItem->modeResize() & RELATIVE_VERT)) {
				nGreedy++;
			}

			sizeMin[i] = pItem->getMinConstrainVert();
			sizeMax[i] = pItem->getMaxConstrainVert();
		}

		// Must not be negative
		availY = max(availY, 0);

		// Then all relative items
		int availRelY = availY;
		int relDiff = 0;
		int relLeft = 0;
		int relCount = 0;
		for(i=0; i<m_paneItems.GetSize(); ++i) {
			CPaneBase* pItem = m_paneItems[i];
			if(pItem->modeResize() & RELATIVE_VERT) {
				int nSizeRelMin = sizeMin[i];
				int nSizeRelMax = sizeMax[i];
				int nSizeRel	= pItem->getConstrainVert(availRelY);

				if( nSizeRel < nSizeRelMin) {
					sizePrimary[i]	= -nSizeRelMin;
					relDiff += ( nSizeRelMin - nSizeRel );
				}
				else if( nSizeRelMax != -1 && nSizeRel > nSizeRelMax) {
					sizePrimary[i]	= -nSizeRelMax;
					relDiff		+= ( nSizeRelMax - nSizeRel );
				}
				else {
					relLeft	+= ( nSizeRel - nSizeRelMin );
					sizePrimary[i]	= nSizeRel;
					relCount	+= nSizeRel;
				}
				availY	-= nSizeRel;
			}
		}
		if(relLeft < relDiff && availY >= (relDiff-relLeft) ){
			availY -= (relDiff-relLeft);
			relDiff = relLeft;
		}
		ASSERT( availY >= 0 && relLeft >= relDiff);

		// Fixup Relative
		while( relDiff != 0) {
			ASSERT( relCount);
			int relSaved = 0;
			for(i=0; i<m_paneItems.GetSize(); ++i) {
				CPaneBase* pItem = m_paneItems[i];
				if(pItem->modeResize() & RELATIVE_VERT && sizePrimary[i] > 0) {

					bool bLast = false;
					int nDiff = (relDiff * sizePrimary[i]) / relCount;
					if( abs(relDiff) <= 5 ) {
						nDiff = relDiff;
						bLast = true;
					}

					int nNewSize = sizePrimary[i] - nDiff;
				
					if( nNewSize < sizeMin[i] ) {
						relSaved += ( sizePrimary[i] - sizeMin[i] );
						sizePrimary[i] = sizeMin[i];
						relCount-= ( sizePrimary[i] );
					}
					else {
						relSaved += ( sizePrimary[i] - nNewSize );
						sizePrimary[i] = nNewSize;
						if(bLast)
							break;
					}
				}
			}
			ASSERT(relSaved != 0);	// Saved some space in every iteration
			relDiff -= relSaved;
		}
		// Fixup Relative - II
		for(i=0; i<m_paneItems.GetSize(); ++i) {
			CPaneBase* pItem = m_paneItems[i];
			if(pItem->modeResize() & RELATIVE_VERT) {
				if(sizePrimary[i] < 0)
					sizePrimary[i] *= -1;
			}
		}


		// lastly all Greedy items
		int greedyDiff = 0;
		int greedyLeft = 0;
		int greedyCount = 0;
		for(i=0; i<m_paneItems.GetSize(); ++i) {
			CPaneBase* pItem = m_paneItems[i];
			if(!(pItem->modeResize() & ABSOLUTE_VERT) && !(pItem->modeResize() & RELATIVE_VERT)) {

				int nSize		= availY / nGreedy;
				int nSizeMin	= sizeMin[i];
				int nSizeMax	= sizeMax[i];

				if( i == m_paneItems.GetUpperBound() )
					nSize = availY;						// the last gets the rest
//				else if( nSize < nSizeMin)
//					nSize = nSizeMin;

				if( nSize < nSizeMin) {
					sizePrimary[i]	= -nSizeMin;
					greedyDiff		+= ( nSizeMin - nSize );
				}
				else if( nSizeMax != -1 && nSize > nSizeMax) {
					sizePrimary[i]	= -nSizeMax;
					greedyDiff		+= ( nSizeMax - nSize );
				}
				else {
					greedyLeft		+= ( nSize - nSize );
					sizePrimary[i]	= nSize;
					greedyCount		+= nSize;
				}

				availY			-= nSize;

				--nGreedy;
			}
		}
		// Fixup Greedy
		while( greedyDiff != 0 && greedyCount > 0) {
			int greedySaved = 0;
			for(i=0; i<m_paneItems.GetSize(); ++i) {
				CPaneBase* pItem = m_paneItems[i];
				if(!(pItem->modeResize() & ABSOLUTE_VERT) && !(pItem->modeResize() & RELATIVE_VERT) && sizePrimary[i] > 0) {

					bool bLast = false;
					int nDiff = (greedyDiff * sizePrimary[i]) / greedyCount;
					if( abs(greedyDiff) <= 5 ) {
						nDiff = greedyDiff;
						bLast = true;
					}

					int nNewSize = sizePrimary[i] - nDiff;			
					if( nNewSize < sizeMin[i] ) {
						greedySaved += ( sizePrimary[i] - sizeMin[i] );
						sizePrimary[i] = sizeMin[i];
						greedyCount-= ( sizePrimary[i] );
					}
					else {
						greedySaved += ( sizePrimary[i] - nNewSize );
						sizePrimary[i] = nNewSize;
						if(bLast)
							break;
					}
				}
			}
			ASSERT(greedySaved != 0 || greedyCount<=0);	// Saved some space in every iteration
			greedyDiff -= greedySaved;
		}
		// Fixup Greedy - II
		for(i=0; i<m_paneItems.GetSize(); ++i) {
			CPaneBase* pItem = m_paneItems[i];
			if(!(pItem->modeResize() & ABSOLUTE_VERT) && !(pItem->modeResize() & RELATIVE_VERT)) {
				if(sizePrimary[i] < 0)
					sizePrimary[i] *= -1;
			}
		}

		// Now reposition all items:
		int nOffset = rcNewArea.top + m_sizeExtraBorder;
		for(i=0; i<m_paneItems.GetSize(); ++i) {
			CPaneBase* pItem = m_paneItems[i];

			CRect rcPane(rcNewArea.left, nOffset, rcNewArea.right, nOffset+sizePrimary[i]);
			pItem->resizeTo( rcPane );

			nOffset += m_sizeBorder + sizePrimary[i];
		}				
	}
	return true;			
}


/////////////////////////////////////////////////////////////////////////////
// CDialogMgr dialog

#pragma warning(disable: 4355)
CDialogMgr::CDialogMgr(UINT nID, CWnd* pParent /*=NULL*/, LPCTSTR strName /*=NULL*/, bool bGripper /*=true*/)
	: CBaseDialog(nID, pParent), CLayoutMgr( this )
{
	//{{AFX_DATA_INIT(CDialogMgr)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
	m_pRootPane = NULL;
	m_bGripper	= bGripper;

	if(strName)
		m_strRegStore = strName;
}
#pragma warning(default: 4355)

BEGIN_MESSAGE_MAP(CDialogMgr, CBaseDialog)
	//{{AFX_MSG_MAP(CDialogMgr)
	ON_WM_SIZE()
	ON_WM_GETMINMAXINFO()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CDialogMgr message handlers

void CDialogMgr::OnSize(UINT nType, int cx, int cy) 
{
	CBaseDialog::OnSize(nType, cx, cy);

	if( abs(cx) + abs(cy) > 0) {
		// Reposition Size Marker
		if( ::IsWindow(m_wndSizeIcon.GetSafeHwnd()) ) {
			CRect r;
			GetClientRect( r );

			WINDOWPLACEMENT	wpl;
			if(m_wndSizeIcon.GetWindowPlacement(&wpl))
			{
				wpl.flags	=	0;
				wpl.rcNormalPosition.left	=	r.Width() - (wpl.rcNormalPosition.right - wpl.rcNormalPosition.left);
				wpl.rcNormalPosition.right	=	r.Width();
				wpl.rcNormalPosition.top	=	r.Height() - (wpl.rcNormalPosition.bottom - wpl.rcNormalPosition.top);
				wpl.rcNormalPosition.bottom=	r.Height();
				m_wndSizeIcon.SetWindowPlacement(&wpl);
				m_wndSizeIcon.SetWindowPos(&wndBottom, 0,0,0,0, SWP_NOMOVE | SWP_NOSIZE );
			}
		}

		// Re-Layout all controls
		UpdateLayout();
	}

}

void CDialogMgr::OnGetMinMaxInfo(MINMAXINFO FAR* lpMMI) 
{
	if(m_pRootPane) {
		CRect rcClient = GetRect();

		CRect rcWnd;
		GetWindowRect(rcWnd);
		
		// How much do Window and Client differ
		rcWnd-=rcClient;

		// Take into account that there is a border around the rootPane
		lpMMI->ptMinTrackSize = CPoint(m_pRootPane->getMinConstrainHorz() + rcWnd.Width() + 2*m_sizeRootBorders.cx,
			m_pRootPane->getMinConstrainVert() + rcWnd.Height() + 2*m_sizeRootBorders.cy);

		int maxWidth = m_pRootPane->getMaxConstrainHorz();
		int maxHeight = m_pRootPane->getMaxConstrainVert();

		if( maxWidth != -1 ) {
			lpMMI->ptMaxTrackSize.x = maxWidth + rcWnd.Width()+ 2*m_sizeRootBorders.cx;
			lpMMI->ptMaxSize.x = maxWidth + rcWnd.Width()+ 2*m_sizeRootBorders.cx;
		}

		if( maxHeight != -1 ) {
			lpMMI->ptMaxTrackSize.y = maxHeight + rcWnd.Height() + 2*m_sizeRootBorders.cy;
			lpMMI->ptMaxSize.y = maxHeight + rcWnd.Height() + 2*m_sizeRootBorders.cy;
		}
	}
}


CRect CDialogMgr::GetRect() 
{ 
	CRect r; 
	GetClientRect(r);

	if( m_bGripper ) {
		if( ::IsWindow(m_wndSizeIcon.GetSafeHwnd()) ) {
			CRect rcSizeIcon;
			m_wndSizeIcon.GetWindowRect( rcSizeIcon );
			r.bottom -= rcSizeIcon.Height() - m_sizeRootBorders.cy;
		}
	}

	return r; 
}

BOOL CDialogMgr::OnInitDialog() 
{
	CBaseDialog::OnInitDialog();

	if(!m_strRegStore.IsEmpty()) {
		Load(m_strRegStore);
	}	

#ifdef _AUTO_SET_ICON
	SetIcon( AfxGetApp()->LoadIcon(IDR_MAINFRAME) ,FALSE);
#endif
	
	// Sizing icon
	if(m_bGripper && m_bmSizeIcon.LoadOEMBitmap(OBM_SIZE))
	{
		CRect			r;
		WINDOWPLACEMENT	wpl;

		GetClientRect(&r);
		GetWindowPlacement(&wpl);

		CRect	szr(r.Width()-24,r.Height()-24,r.Width(),r.Height());
		if(m_wndSizeIcon.Create(NULL,WS_CHILD|SS_BITMAP,szr,m_pWnd) 
//			&& m_wndSizeIcon.ModifyStyleEx(0,WS_EX_TRANSPARENT) 
		)
		{
			m_wndSizeIcon.SetBitmap(m_bmSizeIcon);

			// now the icon has been resized to the real size of the icon
			if(m_wndSizeIcon.GetWindowPlacement(&wpl))
			{
				wpl.flags	=	0;
				wpl.rcNormalPosition.left	=	r.Width() - (wpl.rcNormalPosition.right - wpl.rcNormalPosition.left);
				wpl.rcNormalPosition.right	=	r.Width();
				wpl.rcNormalPosition.top	=	r.Height() - (wpl.rcNormalPosition.bottom - wpl.rcNormalPosition.top);
				wpl.rcNormalPosition.bottom=	r.Height();
				m_wndSizeIcon.SetWindowPlacement(&wpl);
				m_wndSizeIcon.ShowWindow(SW_SHOW);		// finally - show it
				m_wndSizeIcon.SetWindowPos(&wndBottom, 0,0,0,0, SWP_NOMOVE | SWP_NOSIZE );
			}
		}
	}
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}


BOOL CDialogMgr::DestroyWindow() 
{
	// Store size/position
	if(!m_strRegStore.IsEmpty()) {
		Save(m_strRegStore);
	}	

	// Cleanup
	if(m_pRootPane)
		delete m_pRootPane;

	return CBaseDialog::DestroyWindow();
}

/////////////////////////////////////////////////////////////////////////////
// CDialogMgr dialog

#pragma warning(disable: 4355)
#ifdef CS_HELP
CDialogBarMgr::CDialogBarMgr(UINT nID )
	: CBaseDialogBar( nID ), CLayoutMgr( this )
#else
CDialogBarMgr::CDialogBarMgr()
	: CLayoutMgr( this )
#endif
{
	//{{AFX_DATA_INIT(CDialogBarMgr)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
	m_pRootPane = NULL;
	m_bInitialized = false;
	setRootBorders(0,0);
}
#pragma warning(default: 4355)

BEGIN_MESSAGE_MAP(CDialogBarMgr, CBaseDialogBar)
	//{{AFX_MSG_MAP(CDialogBarMgr)
	ON_WM_SIZE()
	ON_WM_GETMINMAXINFO()
	ON_WM_DESTROY()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CDialogBarMgr message handlers

CSize CDialogBarMgr::CalcDynamicLayout(int nLength, DWORD dwMode)
{
	if( ! m_bInitialized ) {
		Initialize();
		m_bInitialized = true;
	}

	CSize sizeRet =  CBaseDialogBar::CalcDynamicLayout(nLength, dwMode);

	CSize sizeMin = sizeRet;
	CSize sizeMax = sizeRet;

	if(m_pRootPane) {
		CRect rcClient = GetRect();

		CRect rcWnd;
		GetWindowRect(rcWnd);
			
		// How much do Window and Client differ
		CSize sizeDiff( rcWnd.Width() - rcClient.Width(), rcWnd.Height() - rcClient.Height());

		// Take into account that there is a border around the rootPane
		sizeMin = CSize(m_pRootPane->getMinConstrainHorz() + sizeDiff.cx + 2*m_sizeRootBorders.cx,
			m_pRootPane->getMinConstrainVert() + sizeDiff.cy + 2*m_sizeRootBorders.cy);


		int maxWidth = m_pRootPane->getMaxConstrainHorz();
		int maxHeight = m_pRootPane->getMaxConstrainVert();

		if( maxWidth != -1 ) {
			sizeMax.cx = maxWidth + sizeDiff.cy + 2*m_sizeRootBorders.cx;
		}

		if( maxHeight != -1 ) {
			sizeMax.cy = maxHeight + sizeDiff.cy + 2*m_sizeRootBorders.cy;
		}
	}

	sizeRet.cx = min( sizeRet.cx, sizeMax.cx );
	sizeRet.cy = min( sizeRet.cy, sizeMax.cy );

	sizeRet.cx = max( sizeRet.cx, sizeMin.cx );
	sizeRet.cy = max( sizeRet.cy, sizeMin.cy );

	return sizeRet;
}

void CDialogBarMgr::OnSize(UINT nType, int cx, int cy) 
{
	CBaseDialogBar::OnSize(nType, cx, cy);

	if( abs(cx) + abs(cy) > 0) {
		// Re-Layout all controls
		UpdateLayout();
	}

}


CRect CDialogBarMgr::GetRect() 
{ 
	CRect r; 
#ifdef __SBARDLG_H__
	GetInsideRect(r);
#else
	GetClientRect(r);
#endif
	if( IsFloating() )
		r.DeflateRect(4,4);

	return r; 
}


void CDialogBarMgr::OnDestroy() 
{
	// Store size/position on your own!

	// Cleanup
	if(m_pRootPane)
		delete m_pRootPane;

	CBaseDialogBar::OnDestroy();
}



/////////////////////////////////////////////////////////////////////////////
// CFormViewMgr dialog

IMPLEMENT_DYNAMIC(CFormViewMgr, CFormView)

#pragma warning(disable: 4355)
CFormViewMgr::CFormViewMgr(UINT nID, LPCTSTR strName /*=NULL*/)
	: CBaseFormView(nID), CLayoutMgr( this )
{
	//{{AFX_DATA_INIT(CFormViewMgr)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
	m_pRootPane = NULL;

	if(strName)
		m_strRegStore = strName;
}
#pragma warning(default: 4355)

BEGIN_MESSAGE_MAP(CFormViewMgr, CBaseFormView)
	//{{AFX_MSG_MAP(CFormViewMgr)
	ON_WM_SIZE()
	ON_WM_GETMINMAXINFO()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CFormViewMgr message handlers

void CFormViewMgr::OnSize(UINT nType, int cx, int cy) 
{
//	CBaseFormView::OnSize(nType, cx, cy);
	SetScrollSizes(MM_TEXT, CSize(cx,cy));
	if( abs(cx) + abs(cy) > 0) {
		// Re-Layout all controls
		UpdateLayout();
	}
//	MoveWindow(0,0,cx,cy);
}

void CFormViewMgr::OnGetMinMaxInfo(MINMAXINFO FAR* lpMMI) 
{
	if(m_pRootPane) {
		CRect rcClient = GetRect();

		CRect rcWnd;
		GetWindowRect(rcWnd);
		
		// How much do Window and Client differ
		rcWnd-=rcClient;

		// Take into account that there is a border around the rootPane
		lpMMI->ptMinTrackSize = CPoint(m_pRootPane->getMinConstrainHorz() + rcWnd.Width() + 2*m_sizeRootBorders.cx,
			m_pRootPane->getMinConstrainVert() + rcWnd.Height() + 2*m_sizeRootBorders.cy);

		int maxWidth = m_pRootPane->getMaxConstrainHorz();
		int maxHeight = m_pRootPane->getMaxConstrainVert();

		if( maxWidth != -1 ) {
			lpMMI->ptMaxTrackSize.x = maxWidth + rcWnd.Width()+ 2*m_sizeRootBorders.cx;
			lpMMI->ptMaxSize.x = maxWidth + rcWnd.Width()+ 2*m_sizeRootBorders.cx;
		}

		if( maxHeight != -1 ) {
			lpMMI->ptMaxTrackSize.y = maxHeight + rcWnd.Height() + 2*m_sizeRootBorders.cy;
			lpMMI->ptMaxSize.y = maxHeight + rcWnd.Height() + 2*m_sizeRootBorders.cy;
		}
	}
}

CFormViewMgr::~CFormViewMgr() 
{
	// Cleanup
	if(m_pRootPane)
		delete m_pRootPane;
}

